Package: mutscan
Title: Preprocessing and Analysis of Deep Mutational Scanning Data
Version: 1.2.0
Authors@R: 
    c(person(given = "Charlotte",
             family = "Soneson",
             role = c("aut", "cre"),
             email = "charlottesoneson@gmail.com",
             comment = c(ORCID = "0000-0003-3833-2169")),
      person(given = "Michael",
    	       family = "Stadler",
    	       role = c("aut"),
    	       email = "michael.stadler@fmi.ch",
    	       comment = c(ORCID = "0000-0002-2269-4934")),
      person(given = "Friedrich Miescher Institute for Biomedical Research",
             role = "cph"))
Description: Provides functionality for processing and statistical
        analysis of multiplexed assays of variant effect (MAVE) and
        similar data. The package contains functions covering the full
        workflow from raw FASTQ files to publication-ready
        visualizations. A broad range of library designs can be
        processed with a single, unified interface.
Depends: R (>= 4.5.0)
Imports: BiocGenerics, S4Vectors, methods, SummarizedExperiment, Rcpp,
        edgeR (>= 3.42.0), dplyr, Matrix, limma, tidyr, stats, GGally,
        ggplot2, tidyselect (>= 1.2.0), tibble, rlang, grDevices, csaw,
        rmarkdown, xfun, DT, ggrepel, IRanges, utils, DelayedArray,
        tools
Suggests: testthat (>= 3.0.0), BiocStyle, knitr, Biostrings, pwalign,
        plotly, scattermore, BiocManager
SystemRequirements: GNU make
biocViews: GeneticVariability, GenomicVariation, Preprocessing
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
VignetteBuilder: knitr
LinkingTo: Rcpp
Config/testthat/edition: 3
URL: https://github.com/fmicompbio/mutscan
BugReports: https://github.com/fmicompbio/mutscan/issues
Config/pak/sysreqs: cmake make libbz2-dev libicu-dev liblzma-dev
        libuv1-dev libssl-dev xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 13:05:22 UTC
RemoteUrl: https://github.com/bioc/mutscan
RemoteRef: RELEASE_3_23
RemoteSha: 6aa85729d6ab31b4227c979a742e96f6d33dae62
NeedsCompilation: yes
Packaged: 2026-04-29 23:32:07 UTC; root
Author: Charlotte Soneson [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3833-2169>),
  Michael Stadler [aut] (ORCID: <https://orcid.org/0000-0002-2269-4934>),
  Friedrich Miescher Institute for Biomedical Research [cph]
Maintainer: Charlotte Soneson <charlottesoneson@gmail.com>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 23:36:57 UTC; unix
