# Generated by roxygen2: do not edit by hand

export(calcNearestStringDist)
export(calculateFitnessScore)
export(calculateRelativeFC)
export(collapseMutants)
export(collapseMutantsByAA)
export(collapseMutantsBySimilarity)
export(digestFastqs)
export(generateQCReport)
export(groupSimilarSequences)
export(linkMultipleVariants)
export(plotDistributions)
export(plotFiltering)
export(plotMeanDiff)
export(plotPairs)
export(plotTotals)
export(plotVolcano)
export(relabelMutPositions)
export(summarizeExperiment)
importFrom(BiocGenerics,"rownames<-")
importFrom(BiocGenerics,rownames)
importFrom(DT,datatable)
importFrom(DelayedArray,rowsum)
importFrom(GGally,eval_data_col)
importFrom(GGally,ggpairs)
importFrom(IRanges,IntegerList)
importFrom(Matrix,colSums)
importFrom(Matrix,rowMeans)
importFrom(Matrix,rowSums)
importFrom(Rcpp,sourceCpp)
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,unstrsplit)
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(csaw,normOffsets)
importFrom(dplyr,across)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,matches)
importFrom(dplyr,mutate)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(edgeR,DGEList)
importFrom(edgeR,estimateDisp)
importFrom(edgeR,getNormLibSizes)
importFrom(edgeR,glmQLFTest)
importFrom(edgeR,glmQLFit)
importFrom(edgeR,normLibSizes)
importFrom(edgeR,predFC)
importFrom(edgeR,scaleOffset)
importFrom(edgeR,topTags)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,annotate)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_abline)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_continuous)
importFrom(ggplot2,scale_fill_discrete)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_log10)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_log10)
importFrom(ggplot2,stat_density2d)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,ylim)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,colorRamp)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,rgb)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,topTable)
importFrom(limma,voom)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(rlang,.data)
importFrom(rmarkdown,render)
importFrom(stats,cor)
importFrom(stats,setNames)
importFrom(tibble,rownames_to_column)
importFrom(tidyr,gather)
importFrom(tidyr,separate)
importFrom(tidyr,separate_rows)
importFrom(tidyselect,matches)
importFrom(tools,file_ext)
importFrom(utils,globalVariables)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
importFrom(utils,relist)
importFrom(xfun,Rscript_call)
useDynLib(mutscan, .registration = TRUE)
