<img src="man/figures/mutscan.png" align="right" alt="mutscan" width="150"/>

<br>

# mutscan

<br>

<!-- badges: start -->
  [![R-CMD-check](https://github.com/fmicompbio/mutscan/workflows/R-CMD-check/badge.svg)](https://github.com/fmicompbio/mutscan/actions)
  <!-- badges: end -->

`mutscan` is an R package to process, statistically analyse and visualise 
multiplexed assays of variant effect data.
The package is under active development, and the functionality and 
user interface may change. 
For more detailed information about how to run `mutscan`, please consult the 
vignettes. 

## Installation

`mutscan` is available from Bioconductor (from release 3.22) and can be 
installed with:

```r
# install.packages("BiocManager")
BiocManager::install("mutscan")
```

You can also install the development version of `mutscan` from GitHub with:

``` r
# install.packages("devtools")
devtools::install_github("fmicompbio/mutscan")
```
